@@ -20,7 +20,7 @@ fi
 export "PATH=$PATH:$GOPATH/bin"
 
 _run_setup() {
-    req_env_vars SKOPEO_PATH SKOPEO_CI_BRANCH GOSRC BUILDTAGS
+    req_env_vars SKOPEO_PATH SKOPEO_CI_BRANCH GOSRC
 
     project_module=$(go list .)
 
@@ -46,7 +46,23 @@ _run_setup() {
 _run_image_tests() {
     req_env_vars GOPATH GOSRC
 
-    # Tests in this repo. are intended to run as a regular user.
+    # Hacky solution to find test that must be run as root.
+    # This looks for the ensureTestCanCreateImages() test function call and gets the
+    # function name where it is called via git grep,
+    # then trims the line to only show the actual function name and add "^$" around it
+    # since go test commands only accepts a single regex.
+    # Then join all names with "|" with paste to again build up a single regex string
+    # that matches all these names.
+    # With that we don't have to run everything twice and can just run the ones that
+    # actually need to be root.
+    # Note we must run git before we switch/chown to the user because it will error
+    # out otherwise since the file ownership doesn't match.
+    test_filter=$(git grep -h --show-function ensureTestCanCreateImages ./storage |
+                    sed -n 's/func \(Test[[:alnum:]]*\)(.*/^\1\$\$/p' |
+                    paste -sd "|" -)
+    showrun make test "BUILDTAGS='$BUILDTAGS'" "TESTFLAGS=-v -run '$test_filter'" TEST_PACKAGES=./storage
+
+    # Most tests in this repo are intended to run as a regular user.
     ROOTLESS_USER="testuser$RANDOM"
     msg "Setting up rootless user '$ROOTLESS_USER'"
     cd $GOSRC || exit 1
@@ -79,8 +95,13 @@ _run_image_tests() {
     msg "Setup known_hosts for root"
     ssh-keyscan localhost > /root/.ssh/known_hosts \
 
+    msg "Start rekor server as $ROOTLESS_USER"
+    showrun ssh $ROOTLESS_USER@localhost $GOSRC/signature/sigstore/rekor/testdata/start-rekor.sh ci
+    # remove rekor server on function exit
+    trap "ssh $ROOTLESS_USER@localhost $GOSRC/signature/sigstore/rekor/testdata/start-rekor.sh ci remove" RETURN
+
     msg "Executing tests as $ROOTLESS_USER"
-    showrun ssh $ROOTLESS_USER@localhost make -C $GOSRC test "BUILDTAGS='$BUILDTAGS'"
+    showrun ssh $ROOTLESS_USER@localhost make -C $GOSRC test "BUILDTAGS='$BUILDTAGS'" "TESTFLAGS=-v" "REKOR_SERVER_URL='http://127.0.0.1:3000'"
 }
 
 req_env_vars GOSRC
