@@ -9,6 +9,7 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"os"
+	"path/filepath"
 	"strings"
 	"testing"
 
@@ -47,8 +48,9 @@ func TestGetBlobForRemoteLayers(t *testing.T) {
 	cache := memory.New()
 
 	imageSource := createImageSource(t, &types.SystemContext{})
+	defer imageSource.Close()
 	layerInfo := types.BlobInfo{
-		Digest: digest.FromBytes([]byte("Hello world")),
+		Digest: digest.FromString("Hello world"),
 		Size:   -1,
 		URLs: []string{
 			"brokenurl",
@@ -69,6 +71,7 @@ func TestGetBlobForRemoteLayersWithTLS(t *testing.T) {
 	imageSource := createImageSource(t, &types.SystemContext{
 		OCICertPath: "fixtures/accepted_certs",
 	})
+	defer imageSource.Close()
 	cache := memory.New()
 
 	layer, size, err := imageSource.GetBlob(context.Background(), types.BlobInfo{
@@ -85,6 +88,7 @@ func TestGetBlobForRemoteLayersOnTLSFailure(t *testing.T) {
 	imageSource := createImageSource(t, &types.SystemContext{
 		OCICertPath: "fixtures/rejected_certs",
 	})
+	defer imageSource.Close()
 	cache := memory.New()
 	layer, size, err := imageSource.GetBlob(context.Background(), types.BlobInfo{
 		URLs: []string{httpServerAddr},
@@ -138,3 +142,37 @@ func createImageSource(t *testing.T, sys *types.SystemContext) types.ImageSource
 	require.NoError(t, err)
 	return imageSource
 }
+
+func TestGetLocalBlobPath(t *testing.T) {
+	tmpDir := loadFixture(t, "delete_image_multiple_images")
+	ref, err := NewReference(tmpDir, "latest")
+	require.NoError(t, err)
+
+	src, err := ref.NewImageSource(context.Background(), &types.SystemContext{})
+	require.NoError(t, err)
+	defer src.Close()
+
+	// success cases
+	for _, dig := range []digest.Digest{
+		"sha256:a2f798327b3f25e3eff54badcb769953de235e62e3e32051d57a5e66246de4a1",
+		"sha256:557ac7d133b7770216a8101268640edf4e88beab1b4e1e1bfc9b1891a1cab861",
+	} {
+		path, err := GetLocalBlobPath(context.Background(), src, dig)
+		require.NoError(t, err)
+		algo, hash, _ := strings.Cut(string(dig), ":")
+		expect := filepath.Join(tmpDir, "blobs", algo, hash)
+		assert.Equal(t, expect, path)
+	}
+
+	// error cases
+	for _, dig := range []digest.Digest{
+		// Invalid digest must error.
+		"sha256:as",
+		// Valid digest but they don't exist in the oci layout thus they must error.
+		"sha256:abababababababababababababababababababababababababababababababab",
+		"sha512:a2f798327b3f25e3eff54badcb769953de235e62e3e32051d57a5e66246de4a1557ac7d133b7770216a8101268640edf4e88beab1b4e1e1bfc9b1891a1cab861",
+	} {
+		_, err = GetLocalBlobPath(context.Background(), src, dig)
+		require.Error(t, err)
+	}
+}
