@@ -53,7 +53,7 @@ func TestGetManifestDescriptor(t *testing.T) {
 			dir:   "fixtures/name_lookups",
 			image: "a",
 			expectedDescriptor: &imgspecv1.Descriptor{
-				MediaType:   "application/vnd.oci.image.manifest.v1+json",
+				MediaType:   "application/vnd.oci.image.index.v1+json",
 				Digest:      "sha256:aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
 				Size:        1,
 				Annotations: map[string]string{"org.opencontainers.image.ref.name": "a"},
@@ -71,6 +71,28 @@ func TestGetManifestDescriptor(t *testing.T) {
 			},
 			expectedIndex: 1,
 		},
+		{ // A valid reference in a multi-manifest directory
+			dir:   "fixtures/name_lookups",
+			image: "c",
+			expectedDescriptor: &imgspecv1.Descriptor{
+				MediaType:   "application/vnd.docker.distribution.manifest.list.v2+json",
+				Digest:      "sha256:cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc",
+				Size:        3,
+				Annotations: map[string]string{"org.opencontainers.image.ref.name": "c"},
+			},
+			expectedIndex: 2,
+		},
+		{ // A valid reference in a multi-manifest directory
+			dir:   "fixtures/name_lookups",
+			image: "d",
+			expectedDescriptor: &imgspecv1.Descriptor{
+				MediaType:   "application/vnd.docker.distribution.manifest.v2+json",
+				Digest:      "sha256:dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
+				Size:        4,
+				Annotations: map[string]string{"org.opencontainers.image.ref.name": "d"},
+			},
+			expectedIndex: 3,
+		},
 		{ // No entry found
 			dir:                "fixtures/name_lookups",
 			image:              "this-does-not-exist",
@@ -257,19 +279,24 @@ func TestNewIndexReference(t *testing.T) {
 	assert.Equal(t, "", ociRef.image)
 	assert.Equal(t, 9999, ociRef.sourceIndex)
 
-	_, err = NewIndexReference(tmpDir+"/thisparentdoesnotexist/something", 10)
-	assert.Error(t, err)
-
-	// sourceIndex cannot be less than -1
-	_, err = NewIndexReference(tmpDir, -3)
-	assert.Error(t, err)
-
-	_, err = NewIndexReference(tmpDir+"/has:colon", 99)
-	assert.Error(t, err)
+	for _, c := range []struct {
+		dir   string
+		index int
+	}{
+		{tmpDir + "/thisparentdoesnotexist/something", 10},
+		{tmpDir, -1},
+		{tmpDir, -3},
+		{tmpDir + "/has:colon", 99},
+	} {
+		_, err = NewIndexReference(c.dir, c.index)
+		assert.Error(t, err)
+	}
 
 	// Test private newReference
 	_, err = newReference(tmpDir, imageValue, 1)
 	assert.Error(t, err)
+	_, err = newReference(tmpDir, "", -3)
+	assert.Error(t, err)
 }
 
 // refToTempOCI creates a temporary directory and returns an reference to it.
